@@ -1,3 +1,4 @@
+#' @export
 coef.evmSim <- function(object, ...){
     res <- apply(object$param, 2, mean)
 #    names(res) <- c(paste("phi:", colnames(object$X.phi)),
@@ -5,5 +6,3 @@ coef.evmSim <- function(object, ...){
     names(res) <- names(object$map$coefficients)
     res
 }
-coefficients.evmSim <- coef.evmSim
-
