@@ -1,3 +1,4 @@
+#' @export
 print.evmSim <- function(x , print.seed=FALSE, ...){
     print(x$call)
     print(x$map$family, verbose=FALSE)
@@ -17,5 +18,5 @@ print.evmSim <- function(x , print.seed=FALSE, ...){
     m <- coef(x)
     print(m , ...)
 
-    invisible()
-}
\ No newline at end of file
+    invisible(x)
+}
