@@ -1,3 +1,4 @@
+#' @export
 print.predict.mex <- function( x, ... ){
     if ( is.R() ) stdev <- function( x ) sqrt( var( x ) )
 
@@ -21,4 +22,5 @@ print.predict.mex <- function( x, ... ){
 
     cat( paste( "\nConditioned on ", cv, " being above its ", 100*x$pqu, "th percentile.\n\n", sep = "" ) )
     print(res, ...)
-}
\ No newline at end of file
+    invisible(x)
+}
